defmodule Storex.Ecto.SQL.Sandbox do
  def allow(metadata, sandbox) when is_binary(metadata) do
    decode_metadata(metadata)
    |> allow(sandbox)
  end

  def allow(%{repo: repo, owner: owner}, sandbox) do
    sandbox.allow(repo, owner, self())
  end

  def allow(%{}, _sandbox), do: :ok
  def allow(nil, _sandbox), do: :ok

  defp decode_metadata(metadata) do
    metadata
    |> String.split("/")
    |> List.last()
    |> case do
      "BeamMetadata (" <> metadata ->
        metadata
        |> binary_part(0, byte_size(metadata) - 1)
        |> parse_metadata()

      _ ->
        %{}
    end
  end

  defp parse_metadata(metadata) do
    metadata
    |> Base.url_decode64!()
    |> :erlang.binary_to_term()
    |> case do
      {:v1, metadata} -> metadata
      _ -> %{}
    end
  end
end
